# -*-python-*-

__package__    = "s60-simpleapp.py"
__version__    = "1.0"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/s60-simplelibs/"
__cvsversion__ = "$Revision: 1.8 $"
__date__       = "$Date: 2007/05/17 16:50:09 $"
__copyright__  = "Copyright (c) 2006-2007 Aaron Straup Cope. Perl Artistic License."

import e32
import appuifw

class simpleapp :

    def __init__ (self) :

        self.__log__   = u""
        self.__kill__  = False
	self.__poll__  = False
        
        self.__current__  = u""
        self.__activity__ = []

        if self.use_lock() :
            self.__lock__ = e32.Ao_lock()
            appuifw.app.exit_key_handler = self.abort

    #
    #
    #
    
    def abort (self) :
        self.__kill__ = True
        self.__lock__.signal()

    #
    #
    #
    
    def loop (self) :
        if self.setup() :
            self.run()

        if self.use_lock() :
            self.__lock__.wait()

    #
    #
    #
    
    def setup (self) :
        return True
    
    #
    #
    #
    
    def run (self) :
	self.poll()

    #
    #
    #
    
    def poll (self) :

        if self.__poll__ :
       	    self.error("Already polling")
            return False

        self.__poll__ = True

	self.do_poll()
        
        self.__poll__ = False
        self.sleep()

    #
    #
    #
    
    def do_poll (self) :
    	pass

    #
    #
    #
    
    def sleep (self, secs=600) :

        self.record_activity("Sleeping for %s seconds" % secs)

        #
        
        if e32.in_emulator() :
            return
            time.sleep(secs)
            self.poll()
        else :
            timer = e32.Ao_timer()
            timer.after(secs, self.poll)
            self.__timer__ = timer
        
    #
    #
    #
    
    def use_lock (self) :
        if e32.in_emulator() :
            return True

        return False
    
    #
    # "STDOUT"
    #

    def log (self, msg, reset=False) :

        if reset :
            self.__log__ = unicode("")
            
        self.__log__ += "%s\n" % msg
        self.write(unicode(self.__log__))

    #
    #
    #
    
    def write(self, txt) :
        
        if e32.in_emulator() :
            t = appuifw.Text()
            t.write(txt)
            appuifw.app.body = t

        else :
            appuifw.app.body = appuifw.Text(txt)

    #
    #
    #

    def announce (self, msg, type='info') :
        self.note(msg, type, True)

    #
    #
    #
    
    def notice (self, msg) :
        self.note(msg, 'info')

    #
    #
    #
    
    def error (self, msg) :
        self.note(msg, 'error')

    #
    #
    #
    
    def note (self, msg, type, gbl=False) :

        if not e32.in_emulator and gbl :
            appuifw.note(unicode(msg), type, gbl)
        else :
            appuifw.note(unicode(msg), type)

    #
    #
    #
    
    def prompt (self, query, type="text") :

        if e32.in_emulator() :
            return raw_input("%s : " % query)

        return appuifw.query(unicode(query), type)

    #
    #
    #

    def record_activity (self, msg, display=True) :

        self.__activity__.append(unicode("* %s" % msg))

        if display :
            self.display_activity()

    #
    #
    #
    
    def display_activity (self) :

        msg   = "\n".join(self.__activity__)
        reset = True
        
        self.log(msg, reset)

    #
    #
    #
    
    def switch_to_app (self) :
        self.display_app()
        self.setup_app_menu()

    #
    #
    #
    
    def display_app (self) :
        reset = True
        self.log(self.__current__, reset)
    
    #
    #
    #

    def switch_to_activity (self) :
        self.display_activity()
        self.setup_activity_menu()

    #
    #
    #

    def setup_activity_menu (self) :
        pass

    #
    #
    #
    
    def setup_app_menu (self) :
        pass

    #
    #
    #

    def setup_simplestore(self, storename) :

        storepath = storename
        
        if not e32.in_emulator() :
            import sysinfo

            if 'E:' in sysinfo.free_drivespace().keys() :
                storepath = 'e:\\%s' % storename
            else :
                storepath = 'c:\\%s' % storename

        #
        
        try :
            self.__store__ = simplestore(storepath)
        except Exception, e :
            self.error("Failed to open local datastore, %s" % e)
            return False

        return True

    #
    #
    #

    def setup_access_point (self) :

        if e32.in_emulator() :
            return True

        import socket

        try :
            apid = socket.select_access_point()
            apnt = socket.access_point(apid)
            socket.set_default_access_point(apnt)
        except Exception, e :
            self.error("Failed to setup access point : %s" % e)
            return False

        return True
    
    #
    #
    #

    def vibrate (self) :

        try :
        	import misty
        	misty.vibrate(500, 100)                        
        except Exception, e :
	    	return False

    
if __name__ == '__main__' :

	import appuifw

        try :
	   	app = simpleapp()
        	app.loop()
	except Exception, e :
        	appuifw.error(unicode("ARGH : %s" % e), 'error')
