# -*-python-*-

__package__    = "s60-simplegps.py"
__version__    = "1.0"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/s60-simplelibs/"
__cvsversion__ = "$Revision: 1.7 $"
__date__       = "$Date: 2007/05/17 16:50:09 $"
__copyright__  = "Copyright (c) 2006-2007 Aaron Straup Cope. Perl Artistic License."

import socket

def simplegps_bt_enabled () :

      if not e32.in_emulator() :
            return True

      try :
            import lightblue
            return True
      except Exception, e :
            return False
      
def simplegps_selectservice() :

      if e32.in_emulator() :

            if not simplegps_bt_enabled() :
                  return None
            
            import lightblue
            s = lightblue.selectservice()
            addr = s[0]
            service = s[1]
      else :
            addr, services = socket.bt_discover()
            service = services.values()[0]
            
      return (addr, int(service))

#
#
#

class simplegps :

      def __init__ (self, addr, service) :
            self.__addr    = addr
            self.__service = service
            self.__target = (self.__addr, self.__service)
            self.__sock = None

      #
      #
      #
      
      def connect (self) :

            try :

                  if e32.in_emulator() :
                        import lightblue
                        sock = lightblue.socket()
                  else :
                        sock = socket.socket(socket.AF_BT, socket.SOCK_STREAM)
                        
                  sock.connect(self.__target)
                  self.__sock = sock
            except Exception, e :
                  raise Exception("Connect error : %s" % e)                  

      #
      #
      #
      
      def disconnect (self) :
                  
            if not self.__sock :
                  return
          
            self.__sock.close()
            self.__sock = None

      #
      #
      #

      def plot (self) :
            
            try :
                  ln = self.read()
            except Exception, e :
                  raise Exception("Read error : %s" % e)                  
        
            try :
                  return self.parse(ln)
            except Exception, e :
                  raise Exception("Parse error : %s" % e)

      #
      #
      #
      
      def read (self, type="$GPRMC") :
            ln = ""
            
            while 1:
                  char = self.__sock.recv(1)

                  if not char:
                        break

                  ln += char
                  
                  if char == "\n":
                        if ln.startswith(type) :
                              break
                        else :
                              ln = ''
                    
            return ln

      #
      #
      #
      
      def parse (self, gps_data) :

            # Thank you :
            # http://positio.rista.net/en/pys60gps/
            
            # Expect 11 ','-delimiters (NMEA 2.2)
            # Expect 12 ','-delimiters (NMEA 2.3)
            
            lat = ''
            lon = ''
            
            if (gps_data.count(",") == 11) :
                  (GPRMC,hhmmss,status,lat,l1,lon,l2,speed_knots,course,ddmmyy,mvd,mdr_cs)=gps_data.split(",")
            elif (gps_data.count(",") == 12) :
                  (GPRMC,hhmmss,status,lat,l1,lon,l2,speed_knots,course,ddmmyy,mvd,mdr,fix_cs)=gps_data.split(",")
            else:
                  pass

            if (status is 'A' and lat is not "" and lon is not ""):
                  lat, lon = self.parse_latlong(lat, l1, lon, l2)
                
            dt = self.parse_dt(ddmmyy, hhmmss)
                  
            return (dt, lat, lon)

      #
      #
      #
      
      def parse_dt (self, ddmmyy, hhmmss) :
            
            d = (ddmmyy[4:6], ddmmyy[2:4], ddmmyy[0:2])
            t = (hhmmss[0:2], hhmmss[2:4], hhmmss[4:6])
            
            d = "-".join(d)
            t = ":".join(t)
            
            return "20%sT%s" % (d, t)
      
      #
      #
      #
      
      def parse_latlong (self, lat, l1, lon, l2):
            
            try:
                  latitude  = float(lat[0:2]) + (float(lat[2:4] + "." + lat[5:9])/60)
                  longitude = float(lon[0:3]) + (float(lon[3:5] + "." + lon[6:10])/60)
                  
                  if l1 is "S":
                        latitude = -latitude

                  if l2 is "W":
                        longitude = -longitude

                  latitude  = float("%.6f" % latitude)
                  longitude = float("%.6f" % longitude)
            
                  return latitude, longitude
            except:
                  return (None, None)
            
