# -*-python-*-

__package__    = "s60_simplegprecorder.py"
__version__    = "1.0"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/s60-simplelibs/"
__cvsversion__ = "$Revision: 1.2 $"
__date__       = "$Date: 2007/04/11 06:44:45 $"
__copyright__  = "Copyright (c) 2006-2007 Aaron Straup Cope. Perl Artistic License."

# s60_include : */s60-simpleapp/s60-simpleapp.py
# s60_include : */s60-simplestore/s60-simplestore.py
# s60_include : */s60-simplegps/s60-simplegps.py
# s60_include : */s60-simplelogger/s60-simplelogger.py

class simplegpsrecorder (simpleapp) :

    def __init__ (self) :
        simpleapp.__init__(self)

        self.__cfg = None
        self.__gps = None
        self.__logger = None

        self.__poll  = True
        self.__delay = 60

        self.__bt_enabled = simplegps_bt_enabled()        
        self.__pos = ()
        
    #
    #
    #

    def setup (self) :

        if not self.setup_cfg() :
            self.record_activity("Setup failed : config")
            return False

        if not self.setup_gps() :
            self.record_activity("Setup failed : GPS")
            return False

        if not self.setup_logger() :
            self.record_activity("Setup failed : logging")
            return False

        self.setup_menu()
        
        self.record_activity("Setup complete")
        return True

    #
    #
    #
    
    def setup_cfg (self) :

        storepath = 'test'
        
        if not e32.in_emulator() :
            import sysinfo

            if 'E:' in sysinfo.free_drivespace().keys() :
                storepath = 'e:\\gpsr.cfg'
            else :
                storepath = 'c:\\gpsr.cfg'

        try : 
            self.__cfg = simplestore(storepath)
        except Exception, e :
            self.error("Failed to load configs : %s" % e)
            return False
        
        return True
    
    #
    #
    #
    
    def setup_gps (self, force=False) :

        if not self.bt_enabled() :
            return True
        
        try :
            (addr, service) = self.get_device_config(force)            
            self.__gps = simplegps(addr, service)
        except Exception, e :
            self.error("Failed to load device glue : %s" % e)
            return False

        return True

    #
    #
    #
    
    def setup_menu (self) :
        items = self.menu_items()
        appuifw.app.menu = items
        
    #
    #
    #
    
    def menu_items (self) :

        items = []

        if self.__poll :
            items.append((unicode("Pause"), self.pause))
        else :
            items.append((unicode("Resume"), self.resume))

        items.append((unicode("Set delay"), self.set_delay))
        items.append((unicode("Configure GPS device"), self.configure_gps_device))        
        
        return items
    
    #
    #
    #
    
    def configure_gps_device (self) :

        already_paused = self.__poll

        if not already_paused :
            self.pause()
            
        force = True
        
        res = self.setup_gps(force)

        if not already_paused :
            poll = False
            self.resume(poll)

        if res :
            self.record_activity("GPS device settings saved")

        return res
    
    #
    #
    #
    
    def get_device_config (self, force=False) :

        addr = self.__cfg.read('bt_addr')
        service = self.__cfg.read('bt_service')        

        if force or not addr or not service :
            try :
                addr, service = simplegps_selectservice()
            except Exception, e :
                raise Exception("Discovery error : %s" % e)

            try :
                self.__cfg.write('bt_addr', addr)
                self.__cfg.write('bt_service', service)
            except Exception, e :
                self.error("Failed to record device information : %s" % e)
                
        return (addr, int(service))
    
    #
    #
    #

    def define_logroot (self) :
        storepath  = './gpsr'
        
        if not e32.in_emulator() :
            import sysinfo

            if 'E:' in sysinfo.free_drivespace().keys() :
                storepath = 'e:\\gpsr'
            else :
                storepath = 'c:\\gpsr'

        return storepath

    #
    #
    #

    def setup_logger (self) :
        
        self.__logger = simplelogger()
        
        #
        
        storepath = self.define_logroot()
        
        self.__logger.set_logroot(storepath)
        return True
    
    #
    #
    #
    
    def do_poll (self) :

        if not self.__poll :
            return
        
        self.record_activity("Polling GPS device")

        if self.bt_enabled() :
            try :
                self.__gps.connect()
            except Exception, e :
                self.error("SNFU : %s" % e)
                return False

        try :
            (dt, lat, lon) = self.plot()
            self.process_data(dt, lat, lon)        
        except Exception, e :
            self.error("Failed to generate location information : %s" % e)            

        if self.bt_enabled() :
            self.__gps.disconnect()

    #
    #
    #
    
    def plot (self) :
        try :
            (dt, lat, lon) = self.__gps.plot()
            return (dt, lat, lon)
        except Exception, e :
            raise Exception("Plot error : %s" % e)
            
    #
    #
    #
    
    def process_data (self, dt, lat, lon) :
        self.record_activity("process GPS data here")
                
    #
    #
    #

    def sleep (self) :
        delay = self.get_delay()
        simpleapp.sleep(self, delay)

    #
    #
    #
    
    def resume (self, poll=True) :
        self.__poll = True
        self.setup_menu()
        
        if poll :
            self.poll()

        self.record_activity("GPS polling restarted")
        
    #
    #
    #
    
    def pause (self) :
        self.__poll = False
        self.setup_menu()
        self.record_activity("GPS polling paused")
        
    #
    #
    #

    def set_delay (self) :

        delay =  self.prompt(u"Seconds between polling", "text")
        self.__cfg.write('delay', delay)

        n = self.__cfg.read('delay')
        self.record_activity("Set delay to %s seconds" % n)
        
    #
    #
    #
    
    def get_delay (self) :

        n = self.__cfg.read('delay')

        if not n or not int(n) :
            n = self.__delay

        return int(n)

    #
    #
    #
    
    def transfer_logs (self) :

        addr = None
        service = None
        
        try :
            addr, services = socket.bt_obex_discover()
            service = services.values()[0]
        except Exception, e :
            self.error("Discovery error : %s" % e)
            return False

        import os
        import os.path

        logroot  = self.define_logroot()
        filelist = os.listdir(logroot)
        
        for fname in filelist :
            path = os.path.join(logroot, fname)
            self.record_activity("Send %s" % path)
            self.send_log(addr, service, unicode(path))

        self.record_activity("Transfer complete")
            
    #
    #
    #
    
    def send_log (self, addr, service, path) :

        try :
            socket.bt_obex_send_file(addr, service, path)
        except Exception, e :
            self.error("Transfer error : %s" % e)
            return False

        return True
    
    #
    #
    #

    def bt_enabled (self) :
        return self.__bt_enabled

    #
    #
    #

    def logger(self) :
        return self.__logger

    #
    #
    #
    
if __name__ == "__main__" :

    gps = gps_recorder()
    gps.loop()
