# -*-python-*-

__package__    = "s60-simpleimageapp.py"
__version__    = "1.0"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/s60-simplelibs/"
__cvsversion__ = "$Revision: 1.4 $"
__date__       = "$Date: 2007/05/17 16:50:09 $"
__copyright__  = "Copyright (c) 2006-2007 Aaron Straup Cope. Perl Artistic License."

# s60_include : */s60-simpleapp/s60-simpleapp.py
# s60_include : */s60-simplestore/s60-simplestore.py
# s60_include : */s60-simplehttp/s60-simplehttp.py

import os

class simpleimageapp (simpleapp) :

    def __init__ (self) :

        simpleapp.__init__(self)
        self.__currentimage = None
        self.__canvas = None
               
    #
    #
    #

    def resize_image (self, image) :

        cnv = self.app_canvas()
        
        sz = image.size
        cv = cnv.size
        
        if cnv[0] >= sz[0] and cv[1] >= sz[1] :
            return image
        
        dims = self.scalify(sz, cv)
        return image.resize(dims, callback=None, keepaspect=1)

    #
    #
    #
    
    def scalify (self, src, cvs) :

        (ix, iy) = src
        (cx, cy) = cvs
        
        if ix > iy :
            rx = int(cx)
            ry = int(cy * (float(cx) / float(ix)))
        else :
            rx = int(cx * (float(cy) / float(iy)))
            ry = int(cy)
        
        return (rx, ry)

    #
    #
    #
    
    def centering_coords (self, image) :

        cnv = self.app_canvas()
        
        (ix, iy) = image.size
        (cx, cy) = cnv.size

        if ix == cx and iy == cy :
            return (0, 0)
        
        rx = int(float(cx - ix) / 2)
        ry = int(float(cy - iy) / 2)        

        return (rx, ry)
    
    #
    #
    #

    def redraw_canvas (self, warnings=False):

        image = self.current_image()

        if not image :
            return False

        try :
            self.set_canvas(image)
        except Exception, e :
            self.error("Redraw error : %s" % e)
            return False
    
        return True

    #
    #
    #
    
    def set_canvas (self, image) :

        coords = self.centering_coords(image)
        
        try :
            appuifw.app.body = self.app_canvas()
            appuifw.app.body.clear(0)
            appuifw.app.body.blit(image) #, target=coords)
        except Exception, e:
            raise Exception("Canvas error : %s" % e)

        return True
    
    #
    #
    #
    
    def draw_image (self, image) :

        self.__current_image = image
        self.redraw_canvas()
        
        return True
    
    #
    #
    #
    
    def show_image (self, image, resize=True) :

        if resize :
            image = self.resize_image(image)

        if not image :
            return False
        
        self.__currentimage = image
        self.draw_image(image)

    #
    #
    #
    
    def remove_image (self) :
        pass
    
    #
    #
    #
    
    def show_image_from_path (self, path) :
        image = self.open_image(path)

        if not image :
            return False
        
        return self.show_image(image)

    #
    #
    #
    
    def open_image (self, path) :

        try :

            if e32.in_emulator() :
                from PIL import Image
                image = Image.open(path)
            else :
                import graphics
                image = graphics.Image.open(path)
                
        except Exception, e :
            self.error("Failed to open image : %s" % e)
            return False

        return image
    
    #
    #
    #
    
    def show_image_from_url (self, url, unlink=True) :

        try :
            http = simplehttp()
            path = http.getstore(url)
        except Exception, e :
            self.error("Failed to get remote image : %s" % e)
            return False

        ok = True
        
        try :
            self.show_image_from_path(path)
        except Exception, e:
            ok = False

        """
        if unlink :
            os.unlink(path)
        """
        
        return ok

    #
    #
    #

    def current_image (self) :
        return self.__currentimage

    #
    #
    #
    
    def app_canvas (self) :

        if not self.__canvas :

            if e32.in_emulator() :
                cnv = appuifw.Canvas()
            else :
                cnv = appuifw.Canvas() #event_callback=None, redraw_callback=self.redraw_canvas)

            self.__canvas = cnv
            
        return self.__canvas
    
    #
    #
    #

class simpletestapp (simpleimageapp) :

    def setup (self) :

        if not self.setup_access_point() :
            return False

        return simpleimageapp.setup(self)
    
    def run (self) :
        url = 'http://assets1.twitter.com/system/user/profile_image/821/normal/me_2006.jpg'
        self.log("fetch %s" % url)

        try :
            self.show_image_from_url(url)
        except Exception, e :
            self.error(e)

        self.log("Done")
        
if __name__ == '__main__' :

    app = simpletestapp()
    app.loop()
